﻿unit UMain;

interface

uses
	Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
	Dialogs, StdCtrls, ComCtrls, Registry, ActnList;

type
	TForm1 = class(TForm)
		SettingPageControl: TPageControl;
		KnownIDEPackagesTabSheet: TTabSheet;
		KnownIDEAssembliesTabSheet: TTabSheet;
		RegPathDescLabel: TLabel;
		KnownIDEPackagesListView: TListView;
		KnownIDEAssembliesListView: TListView;
		ApplyButton: TButton;
    TabSheet1: TTabSheet;
    KnownIDEPackagesWin32OnlyButton: TButton;
    KnownIDEPackagesFullButton: TButton;
    CodeExplorerSortingCheckBox: TCheckBox;
    ActionList1: TActionList;
    ApplyAction: TAction;
    procedure ApplyActionExecute(Sender: TObject);
    procedure ApplyActionUpdate(Sender: TObject);
    procedure KnownIDEPackagesListViewChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure KnownIDEAssembliesListViewChange(Sender: TObject; Item: TListItem;
			Change: TItemChange);
		procedure CodeExplorerSortingCheckBoxClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
		procedure KnownIDEPackagesFullButtonClick(Sender: TObject);
		procedure KnownIDEPackagesWin32OnlyButtonClick(Sender: TObject);
		procedure FormCreate(Sender: TObject);
	private
		FKnownIDEPackagesChanged: Boolean;
		FKnownIDEAssembliesChanged: Boolean;
		FCodeExplorerChanged: Boolean;
		FRegistry: TRegistry;
		FRootDir: string;
		FJapanease: Boolean;
	protected
		procedure JudgeIDELanguage;
		procedure ReadKnownIDEPackages;
		procedure ReadKnownIDEAssemblies;
		procedure ReadCodeExplorer;
		procedure WriteKnownIDEPackages;
		procedure WriteKnownIDEAssemblies;
		procedure WriteCodeExplorer;
		procedure CheckListView(ListView: TListView; const Items: array of string; Check: Boolean);
		procedure ReadValuesToListView(const Known, Unused: string; ListView: TListView);
		procedure WriteValuesFromListView(const Known, Unused: string; ListView: TListView);
	end;

var
	Form1: TForm1;

const
	BDSRootKey = '\Software\Borland\BDS\3.0';
	RootDirKeyValue = 'RootDir';
	JapaneaseResourceFileName = 'Bin\bds.JP';

	KnownIDEPackagesKey = '\Software\Borland\BDS\3.0\Known IDE Packages';
	UnusedIDEPackagesKey = '\Software\Borland\BDS\3.0\Known IDE Packages (unused)';


	DotNetPackages: array[1..17] of string = (
		'$(BDS)\Bin\cside90.bpl', //CSharp IDE Personality
		'$(BDS)\Bin\delphidotnetide90.bpl', //Delphi for .NET IDE Personality
		'$(BDS)\Bin\vbide90.bpl', //VB IDE Personality
		'$(BDS)\Bin\aspcside90.bpl', //CSharp ASP.NET IDE Package
		'$(BDS)\Bin\aspdelphiide90.bpl', //ASP.NET IDE Package List Manager
		'$(BDS)\Bin\dotnetcoreide90.bpl', //.NET Core IDE Package
		'$(BDS)\Bin\iteidenet90.bpl', //Borland Integrated Translation Environment for .NET
		'$(BDS)\Bin\scide90.bpl', //Visual Studio Converter IDE Package
		'$(BDS)\Bin\vcldotnetdesignide90.bpl', //VCL for .NET designer IDE Package
		'$(BDS)\Bin\DataExplorer90.bpl', //データエクスプローラー
		'$(BDS)\Bin\unittestide90.bpl', //ユニットテスト
		'$(BDS)\Bin\caliberide90.bpl', //Thanks to http://delphi2005.cjb.net/
		'$(BDS)\Bin\soapide90.bpl',
		'$(BDS)\Bin\asmview90.bpl',
		'$(BDS)\Bin\dbkdebugproide90.bpl',
		'$(BDS)\Bin\TGIDE90.BPL',
		'$(BDS)\Bin\compro90.bpl');

	KnownIDEAssembliesKey = '\Software\Borland\BDS\3.0\Known IDE Assemblies';
	UnusedIDEAssembliesKey = '\Software\Borland\BDS\3.0\Known IDE Assemblies (unused)';

	CodeExplorerKey = '\Software\Borland\BDS\3.0\Code Explorer';
	CodeExplorerSortingKeyValue: array[Boolean] of string = ('Sorting', 'ソーティング');
	CodeExplorerSorting: array[Boolean] of string = ('Source', 'Alphabetical');

implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
begin
	FRegistry := TRegistry.Create;
	FRegistry.RootKey := HKEY_CURRENT_USER;
	JudgeIDELanguage;
	ReadKnownIDEPackages;
	ReadKnownIDEAssemblies;
	ReadCodeExplorer;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
	FRegistry.Free;
end;

procedure TForm1.KnownIDEPackagesListViewChange(Sender: TObject; Item: TListItem; Change: TItemChange);
begin
	if Change = ctState then
	begin
		FKnownIDEPackagesChanged := True;
	end
end;

procedure TForm1.KnownIDEPackagesWin32OnlyButtonClick(Sender: TObject);
begin
	CheckListView(KnownIDEPackagesListView, DotNetPackages, False);
	FKnownIDEPackagesChanged := True;
end;

procedure TForm1.KnownIDEPackagesFullButtonClick(Sender: TObject);
begin
	CheckListView(KnownIDEPackagesListView, DotNetPackages, True);
	FKnownIDEPackagesChanged := True;
end;

procedure TForm1.KnownIDEAssembliesListViewChange(Sender: TObject; Item: TListItem; Change: TItemChange);
begin
	if Change = ctState then
	begin
		FKnownIDEAssembliesChanged := True;
	end
end;

procedure TForm1.CodeExplorerSortingCheckBoxClick(Sender: TObject);
begin
	FCodeExplorerChanged := True
end;

procedure TForm1.ApplyActionUpdate(Sender: TObject);
begin
	ApplyAction.Enabled := FKnownIDEPackagesChanged or FKnownIDEAssembliesChanged or FCodeExplorerChanged;
end;

procedure TForm1.ApplyActionExecute(Sender: TObject);
begin
	if FKnownIDEPackagesChanged then
	begin
		FKnownIDEPackagesChanged := False;
		WriteKnownIDEPackages;
	end;
	if FKnownIDEAssembliesChanged then
	begin
		FKnownIDEAssembliesChanged := False;
		WriteKnownIDEAssemblies;
	end;
	if FCodeExplorerChanged then
	begin
		FCodeExplorerChanged := False;
		WriteCodeExplorer;
	end;
end;

procedure TForm1.JudgeIDELanguage;
begin
	FJapanease := False;
	FRegistry.Access := KEY_READ;
	if FRegistry.OpenKeyReadOnly(BDSRootKey) then
	begin
		FRootDir := FRegistry.ReadString(RootDirKeyValue);
		FJapanease := FileExists(IncludeTrailingPathDelimiter(FRootDir) + JapaneaseResourceFileName);
	end;
end;

procedure TForm1.ReadKnownIDEPackages;
begin
	ReadValuesToListView(KnownIDEPackagesKey, UnusedIDEPackagesKey, KnownIDEPackagesListView);
	FKnownIDEPackagesChanged := False;
end;

procedure TForm1.WriteKnownIDEPackages;
begin
	if KnownIDEPackagesListView.Enabled then
	begin
		WriteValuesFromListView(KnownIDEPackagesKey, UnusedIDEPackagesKey, KnownIDEPackagesListView);
	end
end;

procedure TForm1.ReadKnownIDEAssemblies;
begin
	ReadValuesToListView(KnownIDEAssembliesKey, UnusedIDEAssembliesKey, KnownIDEAssembliesListView);
	FKnownIDEAssembliesChanged := False;
end;

procedure TForm1.WriteKnownIDEAssemblies;
begin
	if KnownIDEAssembliesListView.Enabled then
	begin
		WriteValuesFromListView(KnownIDEAssembliesKey, UnusedIDEAssembliesKey, KnownIDEAssembliesListView);
	end
end;

procedure TForm1.ReadCodeExplorer;
begin
	CodeExplorerSortingCheckBox.Checked := True;
	try
		FRegistry.Access := KEY_READ;
		if FRegistry.OpenKeyReadOnly(CodeExplorerKey) then
		begin
			if FRegistry.ReadString(CodeExplorerSortingKeyValue[FJapanease]) = CodeExplorerSorting[False] then
			begin
				CodeExplorerSortingCheckBox.Checked := False;
			end;
		end;
	except
	on ERegistryException do
		CodeExplorerSortingCheckBox.Checked := True;
	end;
	FCodeExplorerChanged := False;
end;

procedure TForm1.WriteCodeExplorer;
var
	Sorting: Boolean;
begin
	try
		FRegistry.Access := KEY_ALL_ACCESS;
		if FRegistry.OpenKey(CodeExplorerKey, True) then
		begin
			Sorting := CodeExplorerSortingCheckBox.Checked;
			FRegistry.WriteString(CodeExplorerSortingKeyValue[FJapanease], CodeExplorerSorting[Sorting]);
		end;
	except
	on E: ERegistryException do
		Application.ShowException(E);
	end;
end;

procedure TForm1.CheckListView(ListView: TListView; const Items: array of string; Check: Boolean);
var
	Item: TListItem;
	ItemName: string;
begin
	for Item in ListView.Items do
	begin
		for ItemName in Items do
		begin
			if AnsiSameText(Item.Caption, ItemName) then
			begin
				Item.Checked := Check;
				Break;
			end;
    end;	
	end;
end;

procedure TForm1.ReadValuesToListView(const Known, Unused: string; ListView: TListView);
var
	Names: TStrings;
	Name, Desc: string;
	Item: TListItem;
begin
	ListView.Clear;
	try
		Names := TStringList.Create;
		try
			FRegistry.Access := KEY_READ;
			if FRegistry.OpenKeyReadOnly(Known) then
			begin
				FRegistry.GetValueNames(Names);
				for Name in Names do
				begin
					Desc := FRegistry.ReadString(Name);
					Item := ListView.Items.Add;
					Item.Caption := Name;
					Item.SubItems.Add(Desc);
					Item.Checked := True;
				end;
			end;
			if FRegistry.OpenKeyReadOnly(Unused) then
			begin
				FRegistry.GetValueNames(Names);
				for Name in Names do
				begin
					Desc := FRegistry.ReadString(Name);
					Item := ListView.Items.Add;
					Item.Caption := Name;
					Item.SubItems.Add(Desc);
					Item.Checked := False;
				end;
			end;
		finally
			Names.Free;
		end
	except
	on ERegistryException do
	begin
		ListView.Clear;
		ListView.Enabled := False;
	end;
	end;
end;

procedure TForm1.WriteValuesFromListView(const Known, Unused: string; ListView: TListView);
var
	Names: TStrings;
	Name, Desc: string;
	Item: TListItem;
begin
	try
		Names := TStringList.Create;
		try
			FRegistry.Access := KEY_ALL_ACCESS;
			if FRegistry.OpenKey(Known, True) then
			begin
				FRegistry.GetValueNames(Names);
				for Name in Names do
				begin
					FRegistry.DeleteValue(Name); {Result: Boolean -> 無視}
				end;
				for Item in ListView.Items do
				begin
					if Item.Checked then
					begin
						Name := Item.Caption;
						Desc := Item.SubItems[0];
						FRegistry.WriteString(Name, Desc);
					end;
				end;
			end;
			if FRegistry.OpenKey(Unused, True) then
			begin
				FRegistry.GetValueNames(Names);
				for Name in Names do
				begin
					FRegistry.DeleteValue(Name); {Result: Boolean -> 無視}
				end;
				for Item in ListView.Items do
				begin
					if not Item.Checked then
					begin
						Name := Item.Caption;
						Desc := Item.SubItems[0];
						FRegistry.WriteString(Name, Desc);
					end;
				end;
			end;
		finally
			Names.Free;
		end
	except
	on E: ERegistryException do
		Application.ShowException(E);
	end;
end;

end.

